/*
| -----------------------------------------------------------------------------
| Original Filename: serial.c  (from eyebot examples)
| Original Author: Thomas Lampart  (lamparts@autolife.informatik.uni-stuttgart.de)
|
| Modified by: Gerd Troeger  (troeger@informatik.uni-leipzig.de)
|
| Description: 
| serial I/O with C functions
| -----------------------------------------------------------------------------
*/

#include "librobi/librobi.h"
#include "libimpro/improc.h"
#include "librobi/protos.h"

#include "keys.h"
#include "lcd.h"
#include "cam.h"
#include "types.h" 
#include "kern.h"
#include "rs232.h"

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

const BYTE bmp_header[54] = {
	0x42,0x4d,0x46,0x3c,	0x00,0x00,0x00,0x00,	0x00,0x00,0x36,0x00,	0x00,0x00,0x28,0x00,
	0x00,0x00,0x52,0x00,	0x00,0x00,0x3e,0x00,	0x00,0x00,0x01,0x00,	0x18,0x00,0x00,0x00,
	0x00,0x00,0x10,0x3c,	0x00,0x00,0x74,0x12,	0x00,0x00,0x74,0x12,	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,	0x00,0x00 };
const BYTE pad_zero[4] = { 0,0,0,0 };

void mainMenu(void)
{
	LCDClear();
	LCDMode(SCROLLING|NOCURSOR);
	LCDMenu("Ech","Tim","Img","END");
	printf("functions:\n 1) serial echo\n 2) send time\n 3) send image\n 4) end\n");
}

void main ()
{
	FILE    *strm;
	char    buf[20];
	char    z;

/* disable input/output buffer */

	setvbuf (stdout,NULL,_IONBF,0); 
	setvbuf (stdin,NULL,_IONBF,0);  

/* clear display and write message */

	LCDClear();
	LCDMode(SCROLLING|NOCURSOR);
	LCDMenu("","","","END");
	printf("init...\n");

	/* No OSInitRS232, the bios settings are used */

	strm=fopen (SER1,"rw+b");

	if (strm==NULL) printf("Init Error!\n");
	else
	{
		setvbuf (strm,NULL,_IONBF,0);
		mainMenu();
		while (1)
		{
			z=KEYRead();
			if (z==KEY1)
			{
				LCDClear();
				LCDMenu("","","","END");
				printf("Echoing serial:\n");

				while (1)
				{
					memset(buf,0,sizeof(buf));
					if((fscanf(strm,"%s",buf))!=-1) printf("%s\n",buf);
					z=KEYRead();
					if (z==KEY4) break;
				}
				printf("aborting\n");
				OSWait(100);
				mainMenu();
			}
			if (z==KEY2)
			{
				time    ostime;

				LCDClear();
				LCDMenu("","","","END");
				printf("Sending time:\n");

				while (1)
				{
					OSGetTime(&ostime.hrs,&ostime.mins,&ostime.secs,&ostime.ticks);
					sprintf(buf,"%.2d:%.2d:%.2d:%.2d\r",ostime.hrs,ostime.mins,ostime.secs,ostime.ticks);
					fprintf(strm,buf);
					printf(buf);
					z=KEYRead();
					if (z==KEY4) break;
				}
				printf("aborting\n");
				OSWait(100);
				mainMenu();
			}
			if (z==KEY3)
			{
				int camera,x,y,cnt;
				colimage colimg;
				BYTE linebuf[3*imagecolumns+2];

				LCDClear();
				LCDMenu("","","","END");
				printf("Image Transfer:\n");

				camera=CAMInit(WIDE);
				if (camera==NOCAM) printf("ERROR: No camera!\n");
				else if (camera==INITERROR) printf("ERROR: CAMInit!\n");
				else if (camera<COLCAM) printf("ERROR: No Color Camera!\n");
				else {
					CAMGetColFrame(&colimg,FALSE);
					printf("sending image..\n");
					cnt=fwrite(bmp_header,1,54,strm);
					for (y=0; y<imagerows; y++)
					{
						int k=imagerows-y-1;
						for (x=0; x<imagecolumns; x++)
						{
							linebuf[3*x]   = colimg[k][x][2];
							linebuf[3*x+1] = colimg[k][x][1];
							linebuf[3*x+2] = colimg[k][x][0];
						}
						linebuf[3*imagecolumns]   = 0;
						linebuf[3*imagecolumns+1] = 0;
						cnt+=fwrite(linebuf,1,3*imagecolumns+2,strm);
						printf("%d\r",cnt);
					}
					printf("%d:done\n",cnt);
				}
				OSWait(100);
				mainMenu();
			}
			if (z==KEY4) break;
		}
	}       
	printf("Exiting..\n(press any key)");
	getchar();
}
